%%%%
%% LBlock_lib
%%%%

function s=create_state(X)
%% Transform an hexadecimal message X in a state to be processed
  for i=1:length(X)
    s(i)=hex2dec(X(i));
  end
end

function s=create_state_bin(X)
%% Transform an hexadecimal message X in a binary message.
%% It is used for the key
  for i=1:length(X)
    s(4*(i-1)+1:4*i)=dec2bin(hex2dec(X(i)), 4);
  end
end

function s=create_state_k(X)
%% Transform an binary state in a state to be processed
%% It is used for the key
  k=1;
  for i=1:4:length(X)-3
    s(k)=bin2dec(X(i:i+3));
    k++;
  end
end

%% Encryption

function s=f(X,K) %% Sboxes
  s0=[14 9 15 0 13 4 10 11 1 2 8 3 7 6 12 5];
  s1=[4 11 14 9 15 13 0 10 7 12 5 6 2 8 1 3];
  s2=[1 14 7 12 15 13 0 6 11 5 9 3 2 4 8 10];
  s3=[7 6 8 11 0 15 3 14 9 10 12 13 5 2 4 1];
  s4=[14 5 15 0 7 2 12 13 1 8 4 9 11 10 6 3];
  s5=[2 13 11 12 15 14 0 9 7 10 6 3 1 8 4 5];
  s6=[11 9 4 14 0 15 10 13 6 12 5 7 3 8 1 2];
  s7=[13 10 15 0 14 4 9 11 2 1 8 3 7 5 12 6];
  t=bitxor(X, K);
  s=[s6(t(2)+1) s4(t(4)+1) s7(t(1)+1) s5(t(3)+1) s2(t(6)+1) s0(t(8)+1) s3(t(5)+1) s1(t(7)+1)];
end

function sk=keyschedule(K, i) %% Encryption keyschedule
  s8=[8 7 14 5 15 13 0 6 11 12 9 10 2 4 1 3];
  s9=[11 5 15 0 7 2 9 13 4 8 1 12 14 10 3 6];
  sk=[K(30:80) K(1:29)];
  sk(1:4)=dec2bin(s9(bin2dec(sk(1:4))+1),4);
  sk(5:8)=dec2bin(s8(bin2dec(sk(5:8))+1),4);
  sk(30:34)=dec2bin(bitxor(bin2dec(sk(30:34)), i),5);
end

function sk=keyschedule_d(K, i) %% Decryption keyschedule
  s8_d=[6 14 12 15 13 3 7 1 0 10 11 8 9 5 2 4];
  s9_d=[3 10 5 14 8 1 15 4 9 6 13 0 11 7 12 2];
  sk=K;
  sk(30:34)=dec2bin(bitxor(bin2dec(sk(30:34)), i),5);
  sk(1:4)=dec2bin(s9_d(bin2dec(sk(1:4))+1),4);
  sk(5:8)=dec2bin(s8_d(bin2dec(sk(5:8))+1),4);
  sk=[sk(52:end) sk(1:51)];
end